import os
import requests
import threading
import psutil

# Todo: Create a better stoppable thread class.
class StoppableThread(threading.Thread):
    """Thread class with a stop() method. The thread itself has to check
    regularly for the stopped() condition."""

    def __init__(self):
        super(StoppableThread, self).__init__()
        self._stop_event = threading.Event()

    def stop(self):
        self._stop_event.set()

    def stopped(self):
        return self._stop_event.is_set()

class Download():
    """
    Class for downloading content from the web more than the speed of light! (jk)
    """
    def __init__(self, url, filename="Default"):
        self._cores = psutil.cpu_count()
        self.url = url
        self.saveas = filename

    def get_cores(self):
        """
        Function to return the number of cores.
        """
        return self._cores

    def _download_chunk(self, headers, start):
        obj = requests.get(self.url,headers,stream = True)
        f = open(self.saveas,"ab+")
        f.seek(start)
        f.write(obj.content)
        f.close()

    def download(self):
        """
        Method for downloading the given content.
        """
        response_obj = requests.head(self.url)
        file_name = self.saveas
        file_size = int(response_obj.headers['content-length'])
        self._file_size = file_size
        if file_size == 0:
            raise ValueError('File Size is zero, hence probably an invalid URL.')

        path = os.path.expanduser('~') + "/Downloads"
        os.chdir(path)
        chunk = file_size // self._cores
        self._chunksize = chunk

        for i in range(self._cores):
            start = chunk*i
            end = start + chunk
            if i == self._cores-1:
                end = file_size
            # print(start,end)
            headers = {'Range': 'bytes=%d-%d' % (start, end)}
            thread = threading.Thread(target=self._download_chunk, args= (headers, start))
            thread.setDaemon(True)
            thread.start()

        main = threading.current_thread()
        for t in threading.enumerate():
            if t is main:
                 continue

    def get_chunk_size():
        return self._chunksize

    def get_filesize():
        return self._file_size



#!/usr/bin/env python
import os
from setuptools import setup, find_packages


# https://packaging.python.org/guides/single-sourcing-package-version/
version = {}
with open(os.path.join("src", "downy", "__init__.py")) as fp:
    exec(fp.read(), version)


# http://blog.ionelmc.ro/2014/05/25/python-packaging/
setup(
    name="downy",
    version=version['__version__'],
    description="Python package for Downloading Files in a blink of the eye",
    author="Shreyas Bapat",
    author_email="contact@shreyasb.com",
    url="https://shreyasb.com/",
    download_url="https://github.com/downy/downy",
    license="MIT",
    keywords=[
        "download", "threads", "idm",
        "download manager", "fast"
    ],
    python_requires=">=3.5",
    install_requires=[
        "requests",
        "threading",
        "psutil",
    ],
    extras_require={
        'dev': [
            "coverage",
            "pytest-cov",
            "pycodestyle",
            "sphinx",
            "alabaster",
            "nbsphinx",
            "ipython>=5.0",
            "jupyter-client",
            "ipykernel",
            "ipywidgets",
        ]
    },
    packages=find_packages('src'),
    package_dir={'': 'src'},
    entry_points={
        'console_scripts': [
            'downy = downy.cli:main'
        ]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Education",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    long_description=open('README.rst', encoding='utf-8').read(),
    include_package_data=True,
    zip_safe=False,
)
